#include <db.h>
#include <pthread.h>
#include <string.h>
#include "../datatypes.h"

/* The database handler itself */
static DB *db;
static pthread_mutex_t db_mutex = PTHREAD_MUTEX_INITIALIZER;

int db_init(void)
{
	int ret;

	ret = db_create(&db, NULL, 0);
	if (ret != 0)
	{
		return -1;
	}

	ret = db->open(db, NULL, "users.db", NULL, DB_BTREE, DB_CREATE, 0);
	if (ret != 0)
	{
		return -1;
	}

	return 0;
}

int db_cleanup(void)
{
	if (db != NULL)
	{
		db->close(db, 0);
	}

	return 0;
}

int get_user(char *login, struct usr_t *user)
{
	int ret;
	size_t login_size = strlen(login) + 1;

	DBT key, data;
	memset(&key, 0, sizeof(DBT));
	memset(&data, 0, sizeof(DBT));

	key.data = login;
	key.size = login_size;

	data.data = user;
	data.ulen = sizeof(struct usr_t);
	data.flags = DB_DBT_USERMEM;

	pthread_mutex_lock(&db_mutex);
	ret = db->get(db, NULL, &key, &data, 0);
	pthread_mutex_unlock(&db_mutex);

	return ret;
}

int put_user(char *login, struct usr_t *user)
{
	int ret;
	size_t login_size = strlen(login) + 1;

	DBT key, data;
	memset(&key, 0, sizeof(DBT));
	memset(&data, 0, sizeof(DBT));

	key.data = login;
	key.size = login_size;

	data.data = user;
	data.size = sizeof(struct usr_t);

	pthread_mutex_lock(&db_mutex);
	ret = db->put(db, NULL, &key, &data, 0);
	pthread_mutex_unlock(&db_mutex);

	return ret;
}

int del_user(char *login)
{
	int ret;
	size_t login_size = strlen(login) + 1;

	DBT key;
	memset(&key, 0, sizeof(DBT));

	key.data = login;
	key.size = login_size;

	pthread_mutex_lock(&db_mutex);
	ret = db->del(db, NULL, &key, 0);
	pthread_mutex_unlock(&db_mutex);

	return ret;
}