#include <stdio.h>
#include <stdlib.h>
#include "../kv_list.h"
#include "../modules.h"

int mod_settings_init(void **);
int print_modules(void **);
int print_packets(void **);
int print_config(void **);

static int enabled = 0;
void (*logger)(int, char*, ...);
const char *(*strevent)(unsigned int);

struct module_func_t functions[4];

struct module_t module_init(void *(*config_get)(char*))
{
	/* 4 functions in total */
	struct module_t module;
	module.func_num = 4;
	module.mod_func = functions;

	/* All these functions will be called during master's initialization */
	functions[0].name = "mod_settings_init";
	functions[0].addr = mod_settings_init;
	functions[0].event = MASTER_START;
	functions[0].param = NULL;
	functions[0].func_params = "config[mod_settings_enable],logger,str_event";

	functions[1].name = "print_modules";
	functions[1].addr = print_modules;
	functions[1].event = MASTER_START;
	functions[1].param = NULL;
	functions[1].func_params = "modules";

	functions[2].name = "print_packets";
	functions[2].addr = print_packets;
	functions[2].event = MASTER_START;
	functions[2].param = NULL;
	functions[2].func_params = "custom_packets";

	functions[3].name = "print_config";
	functions[3].addr = print_config;
	functions[3].event = MASTER_START;
	functions[3].param = NULL;
	functions[3].func_params = "config";

	return module;
}

int mod_settings_init(void **param)
{
	if (param == NULL || param[0] == NULL || param[1] == NULL || param[2] == NULL)
	{
		return -1;
	}

	enabled = strtol(param[0], NULL, 10);
	logger = param[1];
	strevent = param[2];

	return 0;
}

int print_modules(void **params)
{
	if (enabled == 0)
	{
		return 0;
	}

	if (params == NULL)
	{
		return -1;
	}

	if (params[0] == NULL)
	{
		logger(0, "No modules registered!");
		return 0;
	}

	size_t modules_counter = 0;
	kv_list *list = params[0];
	struct module_func_t *func;

	logger(0, "Modules:");
	while (1)
	{
		modules_counter++;
		func = list->value;

		logger(0, "{{");
		logger(0, "  name    => \"%s\"", func->name);
		logger(0, "  addr    => %p", func->addr);
		logger(0, "  event   => %s [%d]", strevent(func->event), func->event);
		logger(0, "  params  => \"%s\"", func->func_params);
		logger(0, "}}");

		if (list->next == NULL) break;
		list = list->next;
	}
	logger(0, "%u module functions in total", modules_counter);

	return 0;
}

int print_packets(void **params)
{
	if (enabled == 0)
	{
		return 0;
	}

	if (params == NULL)
	{
		return -1;
	}

	if (params[0] == NULL)
	{
		logger(0, "No custom packets registered!");
		return 0;
	}

	size_t packets_counter = 0;
	kv_list *list = params[0];
	struct module_func_t *func = NULL;

	logger(0, "Custom packets:");
	while (1)
	{
		packets_counter++;
		func = list->value;

		logger(0, "{{");
		logger(0, "  match  => \"%s\"", list->key);
		logger(0, "  func   => \"%s\" @ %p", func->name, func->addr);
		logger(0, "}}");

		if (list->next == NULL) break;
		list = list->next;
	}
	logger(0, "%u custom packets in total", packets_counter);

	return 0;
}

int print_config(void **params)
{
	if (enabled == 0) return 0;

	if (params == NULL)
	{
		return -1;
	}

	if (params[0] == NULL)
	{
		logger(0, "No config settings registered!");
		return 0;
	}

	size_t config_counter = 0;
	kv_list *list = params[0];

	logger(0, "Config settings:");
	while (1)
	{
		config_counter++;

		logger(0, "{{");
		logger(0, "  key    => \"%s\"", list->key);
		logger(0, "  value  => \"%s\"", list->value);
		logger(0, "}}");

		if (list->next == NULL) break;
		list = list->next;
	}
	logger(0, "%u config settings in total", config_counter);

	return 0;
}