#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include "../datatypes.h"
#include "../modules.h"

int update_status (void **);
int send_status (void **);

struct module_func_t functions[2];

/* number of requests */
unsigned long long int requests = 0;

struct module_t module_init(void *(*config_get)(char*))
{
	struct module_t module;

	/* 2 functions */
	module.func_num = 2;
	module.mod_func = functions;

	/*
	*  update_status will be called on every new connection
	*  to masterserver (even before assigning it to a thread)
	*/
	functions[0].name = "update_status";
	functions[0].addr = update_status;
	functions[0].event = NEW_CONNECTION;
	functions[0].param = NULL;
	functions[0].func_params = NULL;

	/*
	*  send_status is called only if user's request
	*  matches 'param' ("GET /status" here)
	*  The only parameter is client's socket
	*  which is used to send html-page
	*/
	functions[1].name = "send_status";
	functions[1].addr = send_status;
	functions[1].event = CUSTOM_PACKET;
	functions[1].param = "GET /status";
	functions[1].func_params = "client_sock";

	return module;
}

int update_status(void **dummy)
{
	requests++;
	return 0;
}

/* Pretty simple and without unneeded optimizations */
int send_status(void **params)
{
	if (params == NULL || params[0] == NULL) return -1;

	char buf[256] = { 0 };
	char fin_buf[256] = { 0 };

	int sockfd = *(int*)params[0];
	sprintf(buf, "Requests: %llu<br>\n", requests);
	sprintf(fin_buf, "HTTP/1.0 200 OK\nContent-Type: text/html\nContent-Length: %u\n\n%s", strlen(buf), buf);
	send(sockfd, fin_buf, strlen(fin_buf), 0);
	return 0;
}