#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../datatypes.h"
#include "../modules.h"

/* better move it to configs */
#define MAX_SIZE	64*1024

char *strtolower(char *dest, const char *src)
{
	int i;
	for (i=0; src[i] != '\0'; i++)
	{
		if (src[i] >= 'A' && src[i] <= 'Z')
		{
			dest[i] = src[i] + 'a' - 'A';
		}
		else
		{
			dest[i] = src[i];
		}
	}

	dest[i] = '\0';
	return dest;
}

/* things we get from master */
static int (*send_file)(int, char*, char*);
static int (*recv_file)(int, char*, size_t);
static char *skins_folder;

static int (*get_user)(char*, struct usr_t *);
static int (*check_session)(char*, char*);
static void (*logger)(int, char*, ...);

/* our functions */
int mod_skins_init (void **);
int send_skin (void **);
int get_skin (void **);

struct module_func_t functions[3];

struct module_t module_init(void *(*config_get)(char*))
{
	struct module_t module;

	/* 3 functions */
	module.func_num = 3;
	module.mod_func = functions;

	/*
	*/
	functions[0].name = "mod_skins_init";
	functions[0].addr = mod_skins_init;
	functions[0].event = MASTER_START;
	functions[0].param = NULL;
	functions[0].func_params = "config[skins_folder],send_file,recv_file,logger,get_user,check_session";

	/*
	*/
	functions[1].name = "send_skin";
	functions[1].addr = send_skin;
	functions[1].event = CUSTOM_PACKET;
	functions[1].param = "GET /Skins/";
	functions[1].func_params = "main_buf,client_sock";

	/*
	*/
	functions[2].name = "get_skin";
	functions[2].addr = get_skin;
	functions[2].event = CUSTOM_PACKET;
	functions[2].param = "SKINUPLOAD";
	functions[2].func_params = "main_buf,client_sock";

	return module;
}

int mod_skins_init(void **params)
{
	if (params == NULL || params[1] == NULL || params[2] == NULL || params[3] == NULL || params[4] == NULL || params[5] == NULL)
	{
		return -1;
	}

	if (params[0] == NULL)
		skins_folder = "skins";
	else
		skins_folder = params[0];

	send_file = params[1];
	recv_file = params[2];
	logger = params[3];
	get_user = params[4];
	check_session = params[5];

	return 0;
}

int send_skin(void **params)
{
	if (params == NULL || params[0] == NULL || params[1] == NULL)
	{
		return -1;
	}

	int ret;
	char *req = params[0];
	int sockfd = *(int*)params[1];

	char username[LOGIN_MAX_LEN+4+1];
	char login[LOGIN_MAX_LEN+4+1];
	/* enough space for skins_folder + '/' + login + '.png' and '\0' */
	char filename[strlen(skins_folder) + 1 + LOGIN_MAX_LEN + 4 + 1];

	ret = sscanf(req, "GET /Skins/%24s", username);
	strtolower(login, username);

	sprintf(filename, "%s/%s", skins_folder, login);

	ret = send_file(sockfd, filename, NULL);
	if (ret != 0)
	{
		logger(1, "[mod_skins] Sending file \"%s\" failed", filename);
	}
	return 0;
}

int get_skin (void **params)
{
	if (params == NULL || params[0] == NULL || params[1] == NULL)
	{
		return -1;
	}

	int ret;

	char session[SESSION_LEN+1];
	char username[LOGIN_MAX_LEN+1];
	char login[LOGIN_MAX_LEN+1];
	/* enough space for skins_folder + '/' + login + '.png' and '\0' */
	char filename[strlen(skins_folder) + 1 + LOGIN_MAX_LEN + 4 + 1];
	struct usr_t user;

	char *req = params[0];
	int sockfd = *(int*)params[1];

	ret = sscanf(req, "SKINUPLOAD\n&user=%20[^&]&%*[^&]&session=%20[^&]", username, session);
	if (ret != 2)
	{
		return -1;
	}

	strtolower(login, username);
	ret = get_user(login, &user);
	if (ret != 0)
	{
		return -1;
	}

	if (check_session(user.session, session) != 0)
	{
		return -1;
	}

	send(sockfd, "MC_SKINUPACK", strlen("MC_SKINUPACK"), 0);

	sprintf(filename, "%s/%s.png", skins_folder, login);
	ret = recv_file(sockfd, filename, MAX_SIZE);
	if (ret != 0)
	{
		logger(1, "Unable to get user's skin [username = \"%s\", filename = \"%s\"", username, filename);
		return -1;
	}

	/* validate_png_file() */

	return 0;
}